@@ -3,9 +3,4 @@ from . import model_loader
 from . import utils
 from . import tools
 
-__all__ = [
-    "dataset_loader",
-    "model_loader",
-    "tools",
-    "utils"
-]
+__all__ = ["dataset_loader", "model_loader", "tools", "utils"]
