@@ -25,14 +25,11 @@ from .bn_helper import relu_inplace
 BN_MOMENTUM = 0.1
 logger = logging.getLogger(__name__)
 
-class ModuleHelper:
 
+class ModuleHelper:
     @staticmethod
     def BNReLU(num_features, bn_type=None, **kwargs):
-        return nn.Sequential(
-            nn.BatchNorm2d(num_features, **kwargs),
-            nn.ReLU()
-        )
+        return nn.Sequential(nn.BatchNorm2d(num_features, **kwargs), nn.ReLU())
 
     @staticmethod
     def BatchNorm2d(*args, **kwargs):
@@ -41,16 +38,18 @@ class ModuleHelper:
 
 def conv3x3(in_planes, out_planes, stride=1):
     """3x3 convolution with padding"""
-    return nn.Conv2d(in_planes, out_planes, kernel_size=3, stride=stride,
-                     padding=1, bias=False)
+    return nn.Conv2d(
+        in_planes, out_planes, kernel_size=3, stride=stride, padding=1, bias=False
+    )
 
 
 class SpatialGather_Module(nn.Module):
     """
-        Aggregate the context features according to the initial 
+        Aggregate the context features according to the initial
         predicted probability distribution.
         Employ the soft-weighted method to aggregate the context.
     """
+
     def __init__(self, cls_num=0, scale=1):
         super(SpatialGather_Module, self).__init__()
         self.cls_num = cls_num
@@ -60,15 +59,16 @@ class SpatialGather_Module(nn.Module):
         batch_size, c, h, w = probs.size(0), probs.size(1), probs.size(2), probs.size(3)
         probs = probs.view(batch_size, c, -1)
         feats = feats.view(batch_size, feats.size(1), -1)
-        feats = feats.permute(0, 2, 1) # batch x hw x c 
-        probs = F.softmax(self.scale * probs, dim=2)# batch x k x hw
-        ocr_context = torch.matmul(probs, feats)\
-        .permute(0, 2, 1).unsqueeze(3)# batch x k x c
+        feats = feats.permute(0, 2, 1)  # batch x hw x c
+        probs = F.softmax(self.scale * probs, dim=2)  # batch x k x hw
+        ocr_context = (
+            torch.matmul(probs, feats).permute(0, 2, 1).unsqueeze(3)
+        )  # batch x k x c
         return ocr_context
 
 
 class _ObjectAttentionBlock(nn.Module):
-    '''
+    """
     The basic implementation for object context block
     Input:
         N X C X H X W
@@ -79,13 +79,11 @@ class _ObjectAttentionBlock(nn.Module):
         bn_type           : specify the bn type
     Return:
         N X C X H X W
-    '''
-    def __init__(self, 
-                 in_channels, 
-                 key_channels, 
-                 scale=1, 
-                 bn_type=None,
-                 align_corners=True):
+    """
+
+    def __init__(
+        self, in_channels, key_channels, scale=1, bn_type=None, align_corners=True
+    ):
         super(_ObjectAttentionBlock, self).__init__()
         self.scale = scale
         self.in_channels = in_channels
@@ -93,29 +91,65 @@ class _ObjectAttentionBlock(nn.Module):
         self.align_corners = align_corners
         self.pool = nn.MaxPool2d(kernel_size=(scale, scale))
         self.f_pixel = nn.Sequential(
-            nn.Conv2d(in_channels=self.in_channels, out_channels=self.key_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.in_channels,
+                out_channels=self.key_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.key_channels, bn_type=bn_type),
-            nn.Conv2d(in_channels=self.key_channels, out_channels=self.key_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.key_channels,
+                out_channels=self.key_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.key_channels, bn_type=bn_type),
         )
         self.f_object = nn.Sequential(
-            nn.Conv2d(in_channels=self.in_channels, out_channels=self.key_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.in_channels,
+                out_channels=self.key_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.key_channels, bn_type=bn_type),
-            nn.Conv2d(in_channels=self.key_channels, out_channels=self.key_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.key_channels,
+                out_channels=self.key_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.key_channels, bn_type=bn_type),
         )
         self.f_down = nn.Sequential(
-            nn.Conv2d(in_channels=self.in_channels, out_channels=self.key_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.in_channels,
+                out_channels=self.key_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.key_channels, bn_type=bn_type),
         )
         self.f_up = nn.Sequential(
-            nn.Conv2d(in_channels=self.key_channels, out_channels=self.in_channels,
-                kernel_size=1, stride=1, padding=0, bias=False),
+            nn.Conv2d(
+                in_channels=self.key_channels,
+                out_channels=self.in_channels,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=False,
+            ),
             ModuleHelper.BNReLU(self.in_channels, bn_type=bn_type),
         )
 
@@ -131,8 +165,8 @@ class _ObjectAttentionBlock(nn.Module):
         value = value.permute(0, 2, 1)
 
         sim_map = torch.matmul(query, key)
-        sim_map = (self.key_channels**-.5) * sim_map
-        sim_map = F.softmax(sim_map, dim=-1)   
+        sim_map = (self.key_channels ** -0.5) * sim_map
+        sim_map = F.softmax(sim_map, dim=-1)
 
         # add bg context ...
         context = torch.matmul(sim_map, value)
@@ -140,23 +174,27 @@ class _ObjectAttentionBlock(nn.Module):
         context = context.view(batch_size, self.key_channels, *x.size()[2:])
         context = self.f_up(context)
         if self.scale > 1:
-            context = F.interpolate(input=context, size=(h, w), mode='bilinear', align_corners=self.align_corners)
+            context = F.interpolate(
+                input=context,
+                size=(h, w),
+                mode="bilinear",
+                align_corners=self.align_corners,
+            )
 
         return context
 
 
 class ObjectAttentionBlock2D(_ObjectAttentionBlock):
-    def __init__(self, 
-                 in_channels, 
-                 key_channels, 
-                 scale=1, 
-                 bn_type=None,
-                 align_corners=True):
-        super(ObjectAttentionBlock2D, self).__init__(in_channels,
-                                                     key_channels,
-                                                     scale, 
-                                                     bn_type=bn_type,
-                                                     align_corners=align_corners)
+    def __init__(
+        self, in_channels, key_channels, scale=1, bn_type=None, align_corners=True
+    ):
+        super(ObjectAttentionBlock2D, self).__init__(
+            in_channels,
+            key_channels,
+            scale,
+            bn_type=bn_type,
+            align_corners=align_corners,
+        )
 
 
 class SpatialOCR_Module(nn.Module):
@@ -164,26 +202,27 @@ class SpatialOCR_Module(nn.Module):
     Implementation of the OCR module:
     We aggregate the global object representation to update the representation for each pixel.
     """
-    def __init__(self, 
-                 in_channels, 
-                 key_channels, 
-                 out_channels, 
-                 scale=1, 
-                 dropout=0.1, 
-                 bn_type=None,
-                 align_corners=True):
+
+    def __init__(
+        self,
+        in_channels,
+        key_channels,
+        out_channels,
+        scale=1,
+        dropout=0.1,
+        bn_type=None,
+        align_corners=True,
+    ):
         super(SpatialOCR_Module, self).__init__()
-        self.object_context_block = ObjectAttentionBlock2D(in_channels, 
-                                                           key_channels, 
-                                                           scale, 
-                                                           bn_type,
-                                                           align_corners=align_corners)
+        self.object_context_block = ObjectAttentionBlock2D(
+            in_channels, key_channels, scale, bn_type, align_corners=align_corners
+        )
         _in_channels = 2 * in_channels
 
         self.conv_bn_dropout = nn.Sequential(
             nn.Conv2d(_in_channels, out_channels, kernel_size=1, padding=0, bias=False),
             ModuleHelper.BNReLU(out_channels, bn_type=bn_type),
-            nn.Dropout2d(dropout)
+            nn.Dropout2d(dropout),
         )
 
     def forward(self, feats, proxy_feats):
@@ -233,13 +272,14 @@ class Bottleneck(nn.Module):
         super(Bottleneck, self).__init__()
         self.conv1 = nn.Conv2d(inplanes, planes, kernel_size=1, bias=False)
         self.bn1 = nn.BatchNorm2d(planes, momentum=BN_MOMENTUM)
-        self.conv2 = nn.Conv2d(planes, planes, kernel_size=3, stride=stride,
-                               padding=1, bias=False)
+        self.conv2 = nn.Conv2d(
+            planes, planes, kernel_size=3, stride=stride, padding=1, bias=False
+        )
         self.bn2 = nn.BatchNorm2d(planes, momentum=BN_MOMENTUM)
-        self.conv3 = nn.Conv2d(planes, planes * self.expansion, kernel_size=1,
-                               bias=False)
-        self.bn3 = nn.BatchNorm2d(planes * self.expansion,
-                               momentum=BN_MOMENTUM)
+        self.conv3 = nn.Conv2d(
+            planes, planes * self.expansion, kernel_size=1, bias=False
+        )
+        self.bn3 = nn.BatchNorm2d(planes * self.expansion, momentum=BN_MOMENTUM)
         self.relu = nn.ReLU(inplace=relu_inplace)
         self.downsample = downsample
         self.stride = stride
@@ -268,11 +308,21 @@ class Bottleneck(nn.Module):
 
 
 class HighResolutionModule(nn.Module):
-    def __init__(self, num_branches, blocks, num_blocks, num_inchannels,
-                 num_channels, fuse_method, multi_scale_output=True, align_corners=True):
+    def __init__(
+        self,
+        num_branches,
+        blocks,
+        num_blocks,
+        num_inchannels,
+        num_channels,
+        fuse_method,
+        multi_scale_output=True,
+        align_corners=True,
+    ):
         super(HighResolutionModule, self).__init__()
         self._check_branches(
-            num_branches, blocks, num_blocks, num_inchannels, num_channels)
+            num_branches, blocks, num_blocks, num_inchannels, num_channels
+        )
 
         self.num_inchannels = num_inchannels
         self.fuse_method = fuse_method
@@ -281,52 +331,70 @@ class HighResolutionModule(nn.Module):
         self.multi_scale_output = multi_scale_output
 
         self.branches = self._make_branches(
-            num_branches, blocks, num_blocks, num_channels)
+            num_branches, blocks, num_blocks, num_channels
+        )
         self.fuse_layers = self._make_fuse_layers()
         self.relu = nn.ReLU(inplace=relu_inplace)
         self.align_corners = align_corners
 
-    def _check_branches(self, num_branches, blocks, num_blocks,
-                        num_inchannels, num_channels):
+    def _check_branches(
+        self, num_branches, blocks, num_blocks, num_inchannels, num_channels
+    ):
         if num_branches != len(num_blocks):
-            error_msg = 'NUM_BRANCHES({}) <> NUM_BLOCKS({})'.format(
-                num_branches, len(num_blocks))
+            error_msg = "NUM_BRANCHES({}) <> NUM_BLOCKS({})".format(
+                num_branches, len(num_blocks)
+            )
             logger.error(error_msg)
             raise ValueError(error_msg)
 
         if num_branches != len(num_channels):
-            error_msg = 'NUM_BRANCHES({}) <> NUM_CHANNELS({})'.format(
-                num_branches, len(num_channels))
+            error_msg = "NUM_BRANCHES({}) <> NUM_CHANNELS({})".format(
+                num_branches, len(num_channels)
+            )
             logger.error(error_msg)
             raise ValueError(error_msg)
 
         if num_branches != len(num_inchannels):
-            error_msg = 'NUM_BRANCHES({}) <> NUM_INCHANNELS({})'.format(
-                num_branches, len(num_inchannels))
+            error_msg = "NUM_BRANCHES({}) <> NUM_INCHANNELS({})".format(
+                num_branches, len(num_inchannels)
+            )
             logger.error(error_msg)
             raise ValueError(error_msg)
 
-    def _make_one_branch(self, branch_index, block, num_blocks, num_channels,
-                         stride=1):
+    def _make_one_branch(self, branch_index, block, num_blocks, num_channels, stride=1):
         downsample = None
-        if stride != 1 or \
-           self.num_inchannels[branch_index] != num_channels[branch_index] * block.expansion:
+        if (
+            stride != 1
+            or self.num_inchannels[branch_index]
+            != num_channels[branch_index] * block.expansion
+        ):
             downsample = nn.Sequential(
-                nn.Conv2d(self.num_inchannels[branch_index],
-                          num_channels[branch_index] * block.expansion,
-                          kernel_size=1, stride=stride, bias=False),
-                nn.BatchNorm2d(num_channels[branch_index] * block.expansion,
-                            momentum=BN_MOMENTUM),
+                nn.Conv2d(
+                    self.num_inchannels[branch_index],
+                    num_channels[branch_index] * block.expansion,
+                    kernel_size=1,
+                    stride=stride,
+                    bias=False,
+                ),
+                nn.BatchNorm2d(
+                    num_channels[branch_index] * block.expansion, momentum=BN_MOMENTUM
+                ),
             )
 
         layers = []
-        layers.append(block(self.num_inchannels[branch_index],
-                            num_channels[branch_index], stride, downsample))
-        self.num_inchannels[branch_index] = \
-            num_channels[branch_index] * block.expansion
+        layers.append(
+            block(
+                self.num_inchannels[branch_index],
+                num_channels[branch_index],
+                stride,
+                downsample,
+            )
+        )
+        self.num_inchannels[branch_index] = num_channels[branch_index] * block.expansion
         for i in range(1, num_blocks[branch_index]):
-            layers.append(block(self.num_inchannels[branch_index],
-                                num_channels[branch_index]))
+            layers.append(
+                block(self.num_inchannels[branch_index], num_channels[branch_index])
+            )
 
         return nn.Sequential(*layers)
 
@@ -334,8 +402,7 @@ class HighResolutionModule(nn.Module):
         branches = []
 
         for i in range(num_branches):
-            branches.append(
-                self._make_one_branch(i, block, num_blocks, num_channels))
+            branches.append(self._make_one_branch(i, block, num_blocks, num_channels))
 
         return nn.ModuleList(branches)
 
@@ -350,36 +417,59 @@ class HighResolutionModule(nn.Module):
             fuse_layer = []
             for j in range(num_branches):
                 if j > i:
-                    fuse_layer.append(nn.Sequential(
-                        nn.Conv2d(num_inchannels[j],
-                                  num_inchannels[i],
-                                  1,
-                                  1,
-                                  0,
-                                  bias=False),
-                        nn.BatchNorm2d(num_inchannels[i], momentum=BN_MOMENTUM)))
+                    fuse_layer.append(
+                        nn.Sequential(
+                            nn.Conv2d(
+                                num_inchannels[j],
+                                num_inchannels[i],
+                                1,
+                                1,
+                                0,
+                                bias=False,
+                            ),
+                            nn.BatchNorm2d(num_inchannels[i], momentum=BN_MOMENTUM),
+                        )
+                    )
                 elif j == i:
                     fuse_layer.append(None)
                 else:
                     conv3x3s = []
-                    for k in range(i-j):
+                    for k in range(i - j):
                         if k == i - j - 1:
                             num_outchannels_conv3x3 = num_inchannels[i]
-                            conv3x3s.append(nn.Sequential(
-                                nn.Conv2d(num_inchannels[j],
-                                          num_outchannels_conv3x3,
-                                          3, 2, 1, bias=False),
-                                nn.BatchNorm2d(num_outchannels_conv3x3,
-                                            momentum=BN_MOMENTUM)))
+                            conv3x3s.append(
+                                nn.Sequential(
+                                    nn.Conv2d(
+                                        num_inchannels[j],
+                                        num_outchannels_conv3x3,
+                                        3,
+                                        2,
+                                        1,
+                                        bias=False,
+                                    ),
+                                    nn.BatchNorm2d(
+                                        num_outchannels_conv3x3, momentum=BN_MOMENTUM
+                                    ),
+                                )
+                            )
                         else:
                             num_outchannels_conv3x3 = num_inchannels[j]
-                            conv3x3s.append(nn.Sequential(
-                                nn.Conv2d(num_inchannels[j],
-                                          num_outchannels_conv3x3,
-                                          3, 2, 1, bias=False),
-                                nn.BatchNorm2d(num_outchannels_conv3x3,
-                                            momentum=BN_MOMENTUM),
-                                nn.ReLU(inplace=relu_inplace)))
+                            conv3x3s.append(
+                                nn.Sequential(
+                                    nn.Conv2d(
+                                        num_inchannels[j],
+                                        num_outchannels_conv3x3,
+                                        3,
+                                        2,
+                                        1,
+                                        bias=False,
+                                    ),
+                                    nn.BatchNorm2d(
+                                        num_outchannels_conv3x3, momentum=BN_MOMENTUM
+                                    ),
+                                    nn.ReLU(inplace=relu_inplace),
+                                )
+                            )
                     fuse_layer.append(nn.Sequential(*conv3x3s))
             fuse_layers.append(nn.ModuleList(fuse_layer))
 
@@ -407,7 +497,9 @@ class HighResolutionModule(nn.Module):
                     y = y + F.interpolate(
                         self.fuse_layers[i][j](x[j]),
                         size=[height_output, width_output],
-                        mode='bilinear', align_corners=self.align_corners)
+                        mode="bilinear",
+                        align_corners=self.align_corners,
+                    )
                 else:
                     y = y + self.fuse_layers[i][j](x[j])
             x_fuse.append(self.relu(y))
@@ -415,35 +507,37 @@ class HighResolutionModule(nn.Module):
         return x_fuse
 
 
-blocks_dict = {
-    'basic': BasicBlock,
-    'bottleneck': Bottleneck
-}
+blocks_dict = {"basic": BasicBlock, "bottleneck": Bottleneck}
 
 
 class HRNetOCR(nn.Module):
-
-    def __init__(self, extra, n_classes=1, pretrained=None, align_corners=True, bn_momentum=0.1, ocr=None, **kwargs):
+    def __init__(
+        self,
+        extra,
+        n_classes=1,
+        pretrained=None,
+        align_corners=True,
+        bn_momentum=0.1,
+        ocr=None,
+        **kwargs
+    ):
         super(HRNetOCR, self).__init__()
         self.n_classes = n_classes
         self.pretrained = pretrained
         self.align_corners = align_corners
         self.bn_momentum = bn_momentum
-        self.ocr = OmegaConf.create(
-            {
-                "mid_channels": 512,
-                "key_channels": 256,
-                "dropout": 0.05,
-                "scale": 1
-            }
-        ) if ocr is None else ocr
+        self.ocr = (
+            OmegaConf.create(
+                {"mid_channels": 512, "key_channels": 256, "dropout": 0.05, "scale": 1}
+            )
+            if ocr is None
+            else ocr
+        )
 
         # stem net
-        self.conv1 = nn.Conv2d(3, 64, kernel_size=3, stride=2, padding=1,
-                               bias=False)
+        self.conv1 = nn.Conv2d(3, 64, kernel_size=3, stride=2, padding=1, bias=False)
         self.bn1 = nn.BatchNorm2d(64, momentum=self.bn_momentum)
-        self.conv2 = nn.Conv2d(64, 64, kernel_size=3, stride=2, padding=1,
-                               bias=False)
+        self.conv2 = nn.Conv2d(64, 64, kernel_size=3, stride=2, padding=1, bias=False)
         self.bn2 = nn.BatchNorm2d(64, momentum=self.bn_momentum)
         self.relu = nn.ReLU(inplace=relu_inplace)
 
@@ -452,71 +546,90 @@ class HRNetOCR(nn.Module):
         block = blocks_dict[self.stage1_cfg.block]
         num_blocks = self.stage1_cfg.num_blocks[0]
         self.layer1 = self._make_layer(block, 64, num_channels, num_blocks)
-        stage1_out_channel = block.expansion*num_channels
+        stage1_out_channel = block.expansion * num_channels
 
         self.stage2_cfg = extra.stage2
         num_channels = self.stage2_cfg.num_channels
         block = blocks_dict[self.stage2_cfg.block]
         num_channels = [
-            num_channels[i] * block.expansion for i in range(len(num_channels))]
+            num_channels[i] * block.expansion for i in range(len(num_channels))
+        ]
         self.transition1 = self._make_transition_layer(
-            [stage1_out_channel], num_channels)
+            [stage1_out_channel], num_channels
+        )
         self.stage2, pre_stage_channels = self._make_stage(
-            self.stage2_cfg, num_channels)
+            self.stage2_cfg, num_channels
+        )
 
         self.stage3_cfg = extra.stage3
         num_channels = self.stage3_cfg.num_channels
         block = blocks_dict[self.stage3_cfg.block]
         num_channels = [
-            num_channels[i] * block.expansion for i in range(len(num_channels))]
-        self.transition2 = self._make_transition_layer(
-            pre_stage_channels, num_channels)
+            num_channels[i] * block.expansion for i in range(len(num_channels))
+        ]
+        self.transition2 = self._make_transition_layer(pre_stage_channels, num_channels)
         self.stage3, pre_stage_channels = self._make_stage(
-            self.stage3_cfg, num_channels)
+            self.stage3_cfg, num_channels
+        )
 
         self.stage4_cfg = extra.stage4
         num_channels = self.stage4_cfg.num_channels
         block = blocks_dict[self.stage4_cfg.block]
         num_channels = [
-            num_channels[i] * block.expansion for i in range(len(num_channels))]
-        self.transition3 = self._make_transition_layer(
-            pre_stage_channels, num_channels)
+            num_channels[i] * block.expansion for i in range(len(num_channels))
+        ]
+        self.transition3 = self._make_transition_layer(pre_stage_channels, num_channels)
         self.stage4, pre_stage_channels = self._make_stage(
-            self.stage4_cfg, num_channels, multi_scale_output=True)
+            self.stage4_cfg, num_channels, multi_scale_output=True
+        )
 
         last_inp_channels = np.int(np.sum(pre_stage_channels))
         ocr_mid_channels = self.ocr.mid_channels
         ocr_key_channels = self.ocr.key_channels
 
         self.conv3x3_ocr = nn.Sequential(
-            nn.Conv2d(last_inp_channels, ocr_mid_channels,
-                      kernel_size=3, stride=1, padding=1),
+            nn.Conv2d(
+                last_inp_channels, ocr_mid_channels, kernel_size=3, stride=1, padding=1
+            ),
             nn.BatchNorm2d(ocr_mid_channels),
             nn.ReLU(inplace=relu_inplace),
         )
         self.ocr_gather_head = SpatialGather_Module(self.n_classes)
 
-        self.ocr_distri_head = SpatialOCR_Module(in_channels=ocr_mid_channels,
-                                                 key_channels=ocr_key_channels,
-                                                 out_channels=ocr_mid_channels,
-                                                 scale=self.ocr.scale,
-                                                 dropout=self.ocr.dropout,
-                                                 align_corners=self.align_corners
-                                                 )
+        self.ocr_distri_head = SpatialOCR_Module(
+            in_channels=ocr_mid_channels,
+            key_channels=ocr_key_channels,
+            out_channels=ocr_mid_channels,
+            scale=self.ocr.scale,
+            dropout=self.ocr.dropout,
+            align_corners=self.align_corners,
+        )
         self.cls_head = nn.Conv2d(
-            ocr_mid_channels, self.n_classes, kernel_size=1, stride=1, padding=0, bias=True)
+            ocr_mid_channels,
+            self.n_classes,
+            kernel_size=1,
+            stride=1,
+            padding=0,
+            bias=True,
+        )
 
         self.aux_head = nn.Sequential(
-            nn.Conv2d(last_inp_channels, last_inp_channels,
-                      kernel_size=1, stride=1, padding=0),
+            nn.Conv2d(
+                last_inp_channels, last_inp_channels, kernel_size=1, stride=1, padding=0
+            ),
             nn.BatchNorm2d(last_inp_channels),
             nn.ReLU(inplace=relu_inplace),
-            nn.Conv2d(last_inp_channels, self.n_classes,
-                      kernel_size=1, stride=1, padding=0, bias=True)
+            nn.Conv2d(
+                last_inp_channels,
+                self.n_classes,
+                kernel_size=1,
+                stride=1,
+                padding=0,
+                bias=True,
+            ),
         )
-        
-    def _make_transition_layer(
-            self, num_channels_pre_layer, num_channels_cur_layer):
+
+    def _make_transition_layer(self, num_channels_pre_layer, num_channels_cur_layer):
         num_branches_cur = len(num_channels_cur_layer)
         num_branches_pre = len(num_channels_pre_layer)
 
@@ -524,29 +637,40 @@ class HRNetOCR(nn.Module):
         for i in range(num_branches_cur):
             if i < num_branches_pre:
                 if num_channels_cur_layer[i] != num_channels_pre_layer[i]:
-                    transition_layers.append(nn.Sequential(
-                        nn.Conv2d(num_channels_pre_layer[i],
-                                  num_channels_cur_layer[i],
-                                  3,
-                                  1,
-                                  1,
-                                  bias=False),
-                        nn.BatchNorm2d(
-                            num_channels_cur_layer[i], momentum=self.bn_momentum),
-                        nn.ReLU(inplace=relu_inplace)))
+                    transition_layers.append(
+                        nn.Sequential(
+                            nn.Conv2d(
+                                num_channels_pre_layer[i],
+                                num_channels_cur_layer[i],
+                                3,
+                                1,
+                                1,
+                                bias=False,
+                            ),
+                            nn.BatchNorm2d(
+                                num_channels_cur_layer[i], momentum=self.bn_momentum
+                            ),
+                            nn.ReLU(inplace=relu_inplace),
+                        )
+                    )
                 else:
                     transition_layers.append(None)
             else:
                 conv3x3s = []
-                for j in range(i+1-num_branches_pre):
+                for j in range(i + 1 - num_branches_pre):
                     inchannels = num_channels_pre_layer[-1]
-                    outchannels = num_channels_cur_layer[i] \
-                        if j == i-num_branches_pre else inchannels
-                    conv3x3s.append(nn.Sequential(
-                        nn.Conv2d(
-                            inchannels, outchannels, 3, 2, 1, bias=False),
-                        nn.BatchNorm2d(outchannels, momentum=self.bn_momentum),
-                        nn.ReLU(inplace=relu_inplace)))
+                    outchannels = (
+                        num_channels_cur_layer[i]
+                        if j == i - num_branches_pre
+                        else inchannels
+                    )
+                    conv3x3s.append(
+                        nn.Sequential(
+                            nn.Conv2d(inchannels, outchannels, 3, 2, 1, bias=False),
+                            nn.BatchNorm2d(outchannels, momentum=self.bn_momentum),
+                            nn.ReLU(inplace=relu_inplace),
+                        )
+                    )
                 transition_layers.append(nn.Sequential(*conv3x3s))
 
         return nn.ModuleList(transition_layers)
@@ -555,20 +679,22 @@ class HRNetOCR(nn.Module):
         downsample = None
         if stride != 1 or inplanes != planes * block.expansion:
             downsample = nn.Sequential(
-                nn.Conv2d(inplanes, planes * block.expansion,
-                          kernel_size=1, stride=stride, bias=False),
+                nn.Conv2d(
+                    inplanes,
+                    planes * block.expansion,
+                    kernel_size=1,
+                    stride=stride,
+                    bias=False,
+                ),
                 nn.BatchNorm2d(planes * block.expansion, momentum=self.bn_momentum),
             )
 
         layers = [block(inplanes, planes, stride, downsample)]
         inplanes = planes * block.expansion
-        layers += [
-            block(inplanes, planes) for _ in range(1, blocks)
-        ]
+        layers += [block(inplanes, planes) for _ in range(1, blocks)]
         return nn.Sequential(*layers)
 
-    def _make_stage(self, layer_config, num_inchannels,
-                    multi_scale_output=True):
+    def _make_stage(self, layer_config, num_inchannels, multi_scale_output=True):
         num_modules = layer_config.num_modules
         num_branches = layer_config.num_branches
         num_blocks = layer_config.num_blocks
@@ -579,18 +705,20 @@ class HRNetOCR(nn.Module):
         modules = []
         for i in range(num_modules):
             # multi_scale_output is only used last module
-            reset_multi_scale_output = False \
-                if not multi_scale_output and i == num_modules - 1 \
-                else True
+            reset_multi_scale_output = (
+                False if not multi_scale_output and i == num_modules - 1 else True
+            )
             modules.append(
-                HighResolutionModule(num_branches,
-                                     block,
-                                     num_blocks,
-                                     num_inchannels,
-                                     num_channels,
-                                     fuse_method,
-                                     reset_multi_scale_output,
-                                     align_corners=self.align_corners)
+                HighResolutionModule(
+                    num_branches,
+                    block,
+                    num_blocks,
+                    num_inchannels,
+                    num_channels,
+                    fuse_method,
+                    reset_multi_scale_output,
+                    align_corners=self.align_corners,
+                )
             )
             num_inchannels = modules[-1].get_num_inchannels()
 
@@ -637,12 +765,15 @@ class HRNetOCR(nn.Module):
 
         # Upsampling
         x0_h, x0_w = x[0].size(2), x[0].size(3)
-        x1 = F.interpolate(x[1], size=(x0_h, x0_w),
-                        mode='bilinear', align_corners=self.align_corners)
-        x2 = F.interpolate(x[2], size=(x0_h, x0_w),
-                        mode='bilinear', align_corners=self.align_corners)
-        x3 = F.interpolate(x[3], size=(x0_h, x0_w),
-                        mode='bilinear', align_corners=self.align_corners)
+        x1 = F.interpolate(
+            x[1], size=(x0_h, x0_w), mode="bilinear", align_corners=self.align_corners
+        )
+        x2 = F.interpolate(
+            x[2], size=(x0_h, x0_w), mode="bilinear", align_corners=self.align_corners
+        )
+        x3 = F.interpolate(
+            x[3], size=(x0_h, x0_w), mode="bilinear", align_corners=self.align_corners
+        )
 
         feats = torch.cat([x[0], x1, x2, x3], 1)
 
@@ -656,15 +787,12 @@ class HRNetOCR(nn.Module):
 
         out = self.cls_head(feats)
 
-        return {
-            'aux' : out_aux,
-            'out': out
-        }
+        return {"aux": out_aux, "out": out}
 
-    def init_weights(self, pretrained='',):
-        logger.info('=> init weights from normal distribution')
+    def init_weights(self, pretrained=""):
+        logger.info("=> init weights from normal distribution")
         for name, m in self.named_modules():
-            if any(part in name for part in {'cls', 'aux', 'ocr'}):
+            if any(part in name for part in {"cls", "aux", "ocr"}):
                 # print('skipped', name)
                 continue
             if isinstance(m, nn.Conv2d):
@@ -673,18 +801,22 @@ class HRNetOCR(nn.Module):
                 nn.init.constant_(m.weight, 1)
                 nn.init.constant_(m.bias, 0)
         if os.path.isfile(pretrained):
-            pretrained_dict = torch.load(pretrained, map_location={'cuda:0': 'cpu'})
-            logger.info('=> loading pretrained model {}'.format(pretrained))
+            pretrained_dict = torch.load(pretrained, map_location={"cuda:0": "cpu"})
+            logger.info("=> loading pretrained model {}".format(pretrained))
             model_dict = self.state_dict()
-            pretrained_dict = {k.replace('last_layer', 'aux_head').replace('model.', ''): v for k, v in pretrained_dict.items()}  
-            print(set(model_dict) - set(pretrained_dict))            
-            print(set(pretrained_dict) - set(model_dict))            
-            pretrained_dict = {k: v for k, v in pretrained_dict.items()
-                               if k in model_dict.keys()}
+            pretrained_dict = {
+                k.replace("last_layer", "aux_head").replace("model.", ""): v
+                for k, v in pretrained_dict.items()
+            }
+            # print(set(model_dict) - set(pretrained_dict))
+            # print(set(pretrained_dict) - set(model_dict))
+            pretrained_dict = {
+                k: v for k, v in pretrained_dict.items() if k in model_dict.keys()
+            }
             # for k, _ in pretrained_dict.items():
-                # logger.info(
-                #     '=> loading {} pretrained model {}'.format(k, pretrained))
+            # logger.info(
+            #     '=> loading {} pretrained model {}'.format(k, pretrained))
             model_dict.update(pretrained_dict)
             self.load_state_dict(model_dict)
         elif pretrained:
-            raise RuntimeError('No such file {}'.format(pretrained))
+            raise RuntimeError("No such file {}".format(pretrained))
