@@ -19,8 +19,13 @@
  ****
 """
 import logging
-from pytorch_segmentation_models_trainer.custom_callbacks.training_callbacks import FrameFieldComputeWeightNormLossesCallback
-from pytorch_segmentation_models_trainer.model_loader.frame_field_model import FrameFieldSegmentationPLModel
+from pytorch_segmentation_models_trainer.custom_callbacks.training_callbacks import (
+    FrameFieldComputeWeightNormLossesCallback,
+)
+from pytorch_segmentation_models_trainer.model_loader.model import Model
+from pytorch_segmentation_models_trainer.model_loader.frame_field_model import (
+    FrameFieldSegmentationPLModel,
+)
 
 import hydra
 from hydra.utils import instantiate
@@ -28,11 +33,11 @@ from omegaconf import DictConfig, OmegaConf
 from pytorch_lightning import Trainer
 
 from pytorch_segmentation_models_trainer.model_loader.model import Model
-from pytorch_segmentation_models_trainer.utils.os_utils import \
-    import_module_from_cfg
+from pytorch_segmentation_models_trainer.utils.os_utils import import_module_from_cfg
 
 logger = logging.getLogger(__name__)
 
+
 @hydra.main()
 def train(cfg: DictConfig):
     """Trains the model.
@@ -44,11 +49,26 @@ def train(cfg: DictConfig):
     """
     logger.info(
         "Starting the training of a model with the following configuration: \n%s",
-        OmegaConf.to_yaml(cfg)
+        OmegaConf.to_yaml(cfg),
+    )
+    if "resume_from_checkpoint" in cfg.hyperparameters:
+        model = import_module_from_cfg(cfg.pl_model).load_from_checkpoint(
+            cfg.hyperparameters.resume_from_checkpoint, cfg=cfg
+        )
+    else:
+        model = (
+            Model(cfg)
+            if "pl_model" not in cfg
+            else import_module_from_cfg(cfg.pl_model)(cfg)
+        )
+    trainer_logger = (
+        instantiate(cfg.logger, _recursive_=False) if "logger" in cfg else True
+    )
+    callback_list = (
+        [instantiate(i, _recursive_=False) for i in cfg.callbacks]
+        if "callbacks" in cfg
+        else []
     )
-    model = Model(cfg) if "pl_model" not in cfg else import_module_from_cfg(cfg.pl_model)(cfg)
-    trainer_logger = instantiate(cfg.logger) if "logger" in cfg else True
-    callback_list = [instantiate(i) for i in cfg.callbacks] if "callbacks" in cfg else []
     if isinstance(model, FrameFieldSegmentationPLModel):
         is_norm_loss_added = False
         for callback in callback_list:
@@ -56,12 +76,11 @@ def train(cfg: DictConfig):
                 is_norm_loss_added = True
                 break
         if not is_norm_loss_added:
-            callback_list.append(
-                FrameFieldComputeWeightNormLossesCallback()
-            )
+            callback_list.append(FrameFieldComputeWeightNormLossesCallback())
     trainer = Trainer(**cfg.pl_trainer, logger=trainer_logger, callbacks=callback_list)
     trainer.fit(model)
     return trainer
 
-if __name__=="__main__":
+
+if __name__ == "__main__":
     train()
