@@ -5,7 +5,7 @@
                               -------------------
         begin                : 2021-03-31
         git sha              : $Format:%H$
-        copyright            : (C) 2021 by Philipe Borba - Cartographic Engineer 
+        copyright            : (C) 2021 by Philipe Borba - Cartographic Engineer
                                                             @ Brazilian Army
         email                : philipeborba at gmail dot com
  ***************************************************************************/
@@ -24,32 +24,40 @@
 import numpy as np
 import torch
 from pytorch_segmentation_models_trainer.utils.complex_utils import (
-    complex_abs_squared, complex_mul, complex_sqrt)
+    complex_abs_squared,
+    complex_mul,
+    complex_sqrt,
+)
 from torch.nn import functional as F
 
+
 class LaplacianPenalty:
     def __init__(self, channels: int):
         self.channels = channels
-        self.filter = torch.tensor([[0.5, 1.0, 0.5],
-                                    [1.0, -6., 1.0],
-                                    [0.5, 1.0, 0.5]]) / 12
-        self.filter = self.filter[None, None, ...].expand(
-            self.channels, -1, -1, -1)
+        self.filter = (
+            torch.tensor([[0.5, 1.0, 0.5], [1.0, -6.0, 1.0], [0.5, 1.0, 0.5]]) / 12
+        )
+        self.filter = self.filter[None, None, ...].expand(self.channels, -1, -1, -1)
 
     def laplacian_filter(self, tensor):
-        penalty_tensor = F.conv2d(tensor, self.filter.to(tensor.device), padding=1,
-                                  groups=self.channels)
+        penalty_tensor = F.conv2d(
+            tensor, self.filter.to(tensor.device), padding=1, groups=self.channels
+        )
         return torch.abs(penalty_tensor)
 
     def __call__(self, tensor: torch.Tensor) -> torch.Tensor:
         return self.laplacian_filter(tensor)
 
+
 def framefield_align_error(c0, c2, z, complex_dim=-1):
-    assert c0.shape == c2.shape == z.shape, \
-        "All inputs should have the same shape. Currently c0: {}, c2: {}, z: {}".format(
-            c0.shape, c2.shape, z.shape)
-    assert c0.shape[complex_dim] == c2.shape[complex_dim] == z.shape[complex_dim] == 2, \
-        "All inputs should have their complex_dim size equal 2 (real and imag parts)"
+    assert (
+        c0.shape == c2.shape == z.shape
+    ), "All inputs should have the same shape. Currently c0: {}, c2: {}, z: {}".format(
+        c0.shape, c2.shape, z.shape
+    )
+    assert (
+        c0.shape[complex_dim] == c2.shape[complex_dim] == z.shape[complex_dim] == 2
+    ), "All inputs should have their complex_dim size equal 2 (real and imag parts)"
 
     z_squared = complex_mul(z, z, complex_dim=complex_dim)
     z_pow_4 = complex_mul(z_squared, z_squared, complex_dim=complex_dim)
@@ -59,12 +67,12 @@ def framefield_align_error(c0, c2, z, complex_dim=-1):
     loss = complex_abs_squared(f_z, complex_dim)
     return loss
 
+
 def c0c2_to_uv(c0c2: torch.Tensor) -> torch.Tensor:
     c0, c2 = torch.chunk(c0c2, 2, dim=1)
     c2_squared = complex_mul(c2, c2, complex_dim=1)
     c2_squared_minus_4c0 = c2_squared - 4 * c0
-    sqrt_c2_squared_minus_4c0 = complex_sqrt(
-        c2_squared_minus_4c0, complex_dim=1)
+    sqrt_c2_squared_minus_4c0 = complex_sqrt(c2_squared_minus_4c0, complex_dim=1)
     u_squared = (c2 + sqrt_c2_squared_minus_4c0) / 2
     v_squared = (c2 - sqrt_c2_squared_minus_4c0) / 2
     # Shape (B, 'uv': 2, 'complex': 2, H, W)
@@ -72,6 +80,7 @@ def c0c2_to_uv(c0c2: torch.Tensor) -> torch.Tensor:
     uv = complex_sqrt(uv_squared, complex_dim=2)
     return uv
 
+
 def compute_closest_in_uv(directions: torch.Tensor, uv: torch.Tensor) -> torch.Tensor:
     """
     For each direction, compute if it is more aligned with {u, -u} (output 0) or {v, -v} (output 1).
@@ -87,6 +96,7 @@ def compute_closest_in_uv(directions: torch.Tensor, uv: torch.Tensor) -> torch.T
 
     return closest_in_uv
 
+
 def detect_corners(polylines, u, v):
     def compute_direction_score(ij, edges, field_dir):
         values = field_dir[ij[:, 0], ij[:, 1]]
@@ -117,10 +127,13 @@ def detect_corners(polylines, u, v):
         if np.max(np.abs(polyline[0] - polyline[-1])) < 1e-6:
             # Closed polyline
             left_edges = np.concatenate(
-                [polyline[-2:-1] - polyline[-1:], polyline[:-2] - polyline[1:-1]], axis=0)
+                [polyline[-2:-1] - polyline[-1:], polyline[:-2] - polyline[1:-1]],
+                axis=0,
+            )
             right_edges = polyline[1:] - polyline[:-1]
             corner_mask[:-1] = compute_is_corner(
-                polyline[:-1, :], left_edges, right_edges)
+                polyline[:-1, :], left_edges, right_edges
+            )
             # left_edges and right_edges do not include the redundant last vertex, thus we have to do this assignment:
             corner_mask[-1] = corner_mask[0]
         else:
@@ -130,24 +143,29 @@ def detect_corners(polylines, u, v):
             left_edges = polyline[:-2] - polyline[1:-1]
             right_edges = polyline[2:] - polyline[1:-1]
             corner_mask[1:-1] = compute_is_corner(
-                polyline[1:-1, :], left_edges, right_edges)
+                polyline[1:-1, :], left_edges, right_edges
+            )
         corner_masks.append(corner_mask)
 
     return corner_masks
 
+
 def compute_crossfield_to_plot(gt_crossfield_angle, crossfield_shape=None):
     u_angle = gt_crossfield_angle
-    v_angle = gt_crossfield_angle + np.pi / 2 
+    v_angle = gt_crossfield_angle + np.pi / 2
     u = np.cos(u_angle) + 1j * np.sin(u_angle)
     v = np.cos(v_angle) + 1j * np.sin(v_angle)
     c0 = np.power(u, 2) * np.power(v, 2)
-    c2 = - (np.power(u, 2) + np.power(v, 2))
+    c2 = -(np.power(u, 2) + np.power(v, 2))
 
-    crossfield = torch.zeros((1, 4, u_angle.shape[-2], u_angle.shape[-1])) if crossfield_shape is None \
+    crossfield = (
+        torch.zeros((1, 4, u_angle.shape[-2], u_angle.shape[-1]))
+        if crossfield_shape is None
         else torch.zeros(crossfield_shape)
+    )
 
     crossfield[:, 0, :, :] = c0.real
     crossfield[:, 1, :, :] = c0.imag
     crossfield[:, 2, :, :] = c2.real
     crossfield[:, 3, :, :] = c2.imag
-    return crossfield
\ No newline at end of file
+    return crossfield
